@@ -8,6 +8,7 @@ import org.apache.commons.lang3.tuple.Pair;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 import org.broadinstitute.barclay.argparser.*;
+import org.broadinstitute.barclay.utils.Utils;
 
 import java.lang.reflect.*;
 import java.util.*;
@@ -304,7 +305,7 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
         if (clp != null) {
             // do positional arguments, followed by named arguments
             processPositionalArguments(clp, argMap);
-            clp.getArgumentDefinitions().stream().forEach(argDef -> processNamedArgument(currentWorkUnit, argMap, argDef));
+            clp.getNamedArgumentDefinitions().stream().forEach(argDef -> processNamedArgument(currentWorkUnit, argMap, argDef));
 
             // sort the resulting args
             argMap.entrySet().stream().forEach( entry -> entry.setValue(sortArguments(entry.getValue())));
@@ -367,82 +368,56 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
         }
     }
 
-    private void processNamedArgument(
+    /**
+     * Add the named argument {@code argDef}to the property map if applicable.
+     * @param currentWorkUnit current work unit
+     * @param args the freemarker arg map
+     * @param argDef the arg to add
+     */
+    protected void processNamedArgument(
             final DocWorkUnit currentWorkUnit,
             final Map<String, List<Map<String, Object>>> args,
-            final CommandLineArgumentParser.ArgumentDefinition argDef)
+            final NamedArgumentDefinition argDef)
     {
-        if (!argDef.isControlledByPlugin() &&
-                (argDef.field.getAnnotation(Hidden.class) == null || getDoclet().showHiddenFeatures())) {
-            // first find the fielddoc for the target
-            FieldDoc fieldDoc = getFieldDocForCommandLineArgument(currentWorkUnit, argDef);
-            final Map<String, Object> argBindings = docForArgument(fieldDoc, argDef);
-            final String kind = docKindOfArg(argDef);
-            argBindings.put("kind", kind);
-
-            // Retrieve default value
-            final Object fieldValue = argDef.getFieldValue();
-            argBindings.put("defaultValue",
-                    fieldValue == null ?
-                            argDef.defaultValue :
-                            prettyPrintValueString(fieldValue));
-
-            if (fieldValue instanceof Number) {
-                // Retrieve min and max / hard and soft value thresholds for numeric args
-                Argument argAnnotation = argDef.field.getAnnotation(Argument.class);
-                argBindings.put("minValue", argAnnotation.minValue());
-                argBindings.put("maxValue", argAnnotation.maxValue());
-                argBindings.put("minRecValue",
-                        argAnnotation.minRecommendedValue() != Double.NEGATIVE_INFINITY ?
-                                argAnnotation.minRecommendedValue() :
-                                "NA");
-                argBindings.put("maxRecValue",
-                        argAnnotation.maxRecommendedValue() != Double.POSITIVE_INFINITY ?
-                                argAnnotation.maxRecommendedValue() :
-                                "NA");
-            } else {
-                argBindings.put("minValue", "NA");
-                argBindings.put("maxValue", "NA");
-                argBindings.put("minRecValue", "NA");
-                argBindings.put("maxRecValue", "NA");
-            }
-
-            // Add in the number of times you can specify it:
-            argBindings.put("minElements", argDef.field.getAnnotation(Argument.class).minElements());
-            argBindings.put("maxElements", argDef.field.getAnnotation(Argument.class).maxElements());
-
-            // if its a plugin descriptor arg, get the allowed values
-            processPluginDescriptorArgument(argDef, argBindings);
+        // Rather than including all the doc for all of the arguments of all plugins right in with the main doc for
+        // a given DocumentedFeature, we instead exclude them (note that this excludes arguments that are
+        // controlled by a plugin, but not by the controlling descriptor itself) by default, on the premise that the
+        // plugins themselves are standalone @DocumentedFeatures that have their own doc. Custom doclets can override
+        // processNamedArgument and provide an alternative policy.
+        if (!argDef.isControlledByPlugin() && (!argDef.isHidden() || getDoclet().showHiddenFeatures())) {
+            final Map<String, Object> argMap = new HashMap<>();
+            final FieldDoc fieldDoc = getFieldDocForCommandLineArgument(currentWorkUnit, argDef);
+            final String argKind = processNamedArgument(argMap, argDef, fieldDoc.commentText());
 
             // Finalize argument bindings
-            args.get(kind).add(argBindings);
-            args.get("all").add(argBindings);
+            args.get(argKind).add(argMap);
+            args.get("all").add(argMap);
         }
     }
 
     private FieldDoc getFieldDocForCommandLineArgument(
             final DocWorkUnit currentWorkUnit,
-            final CommandLineArgumentParser.ArgumentDefinition argDef)
+            final NamedArgumentDefinition argDef)
     {
         // Retrieve the ClassDoc corresponding to this argument directly
-        final String declaringClassTypeName = argDef.field.getDeclaringClass().getTypeName();
+        final String declaringClassTypeName = argDef.getUnderlyingField().getDeclaringClass().getTypeName();
         final ClassDoc declaringClassDoc = getDoclet().getRootDoc().classNamed(declaringClassTypeName);
 
         if (declaringClassDoc == null) {
             throw new DocException(
                     String.format("Can't resolve ClassDoc for declaring class for argument \"%s\" in \"%s\" with qualified name \"%s\"",
-                            argDef.field.getName(),
+                            argDef.getUnderlyingField().getName(),
                             currentWorkUnit.getClassDoc().qualifiedTypeName(),
                             declaringClassTypeName));
         }
 
-        final FieldDoc fieldDoc = getFieldDoc(declaringClassDoc, argDef.field.getName());
+        final FieldDoc fieldDoc = getFieldDoc(declaringClassDoc, argDef.getUnderlyingField().getName());
 
         if (fieldDoc == null) {
             throw new DocException(
                     String.format(
                         "The class \"%s\" is referenced by \"%s\", and must be included in the list of documentation sources.",
-                        argDef.field.getDeclaringClass().getCanonicalName(),
+                        argDef.getUnderlyingField().getDeclaringClass().getCanonicalName(),
                         currentWorkUnit.getClassDoc().qualifiedTypeName())
             );
         }
@@ -453,17 +428,17 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
         if (!fieldDoc.qualifiedName().startsWith(normalizedTypeName)) {
             // The qualified name for a FieldDoc doesn't include the full path of the class if it corresponds to
             // an argument who's declaring class is local/anonymous
-            if (argDef.field.getDeclaringClass().isLocalClass() || argDef.field.getDeclaringClass().isAnonymousClass()) {
+            if (argDef.getUnderlyingField().getDeclaringClass().isLocalClass() || argDef.getUnderlyingField().getDeclaringClass().isAnonymousClass()) {
                 logger.warn(String.format(
                         "Field level Javadoc is ignored for local/anonymous class for member field \"%s\" in \"%s\" of type \"%s\".",
-                        argDef.field.getName(),
+                        argDef.getUnderlyingField().getName(),
                         currentWorkUnit.getClazz().getCanonicalName(),
                         declaringClassTypeName));
             } else {
                 logger.warn(String.format(
                         "Can't validate FieldDoc \"%s\" for member field \"%s\" in \"%s\" of type \"%s\".",
                         fieldDoc.qualifiedName(),
-                        argDef.field.getName(),
+                        argDef.getUnderlyingField().getName(),
                         currentWorkUnit.getClazz().getCanonicalName(),
                         declaringClassTypeName));
             }
@@ -471,23 +446,33 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
         return fieldDoc;
     }
 
-    private void processPositionalArguments(
+    /**
+     * Process any positional arguments for this tool by populating and adding an argument bindings map to the
+     * top level freemarker map for the positional arguments, if they are defined by the tool being processed.
+     * If no positional arguments are defined, do nothing.
+     *
+     * @param clp the instantiated {@link CommandLineArgumentParser} in use for this run. If the current
+     *            command tool has positional arguments, the definition can be retrieved using
+     *            {@link CommandLineArgumentParser#getPositionalArgumentDefinition()}.
+     * @param args the top level freemarker map for this tool, to be populated with an argument map for
+     *            positional args
+     */
+    protected void processPositionalArguments(
             final CommandLineArgumentParser clp,
             final Map<String, List<Map<String, Object>>> args) {
         // first get the positional arguments
-        final Field positionalField = clp.getPositionalArguments();
-        if (positionalField != null) {
+        final PositionalArgumentDefinition positionalArgDef = clp.getPositionalArgumentDefinition();
+        if (positionalArgDef != null) {
             final Map<String, Object> argBindings = new HashMap<>();
-            PositionalArguments posArgs = positionalField.getAnnotation(PositionalArguments.class);
             argBindings.put("kind", "positional");
             argBindings.put("name", NAME_FOR_POSITIONAL_ARGS);
-            argBindings.put("summary", posArgs.doc());
-            argBindings.put("fulltext", posArgs.doc());
+            argBindings.put("summary", positionalArgDef.getPositionalArgumentsAnnotation().doc());
+            argBindings.put("fulltext", positionalArgDef.getPositionalArgumentsAnnotation().doc());
             argBindings.put("otherArgumentRequired", "NA");
             argBindings.put("synonyms", "NA");
             argBindings.put("exclusiveOf", "NA");
-            argBindings.put("type", argumentTypeString(positionalField.getGenericType()));
-            argBindings.put("options", Collections.EMPTY_LIST);
+            argBindings.put("type", argumentTypeString(positionalArgDef.getUnderlyingField().getGenericType()));
+            argBindings.put("options", getPossibleValues(positionalArgDef, "positional"));
             argBindings.put("attributes", "NA");
             argBindings.put("required", "yes");
             argBindings.put("minRecValue", "NA");
@@ -495,58 +480,47 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
             argBindings.put("minValue", "NA");
             argBindings.put("maxValue", "NA");
             argBindings.put("defaultValue", "NA");
-            argBindings.put("minElements", posArgs.minElements());
-            argBindings.put("maxElements", posArgs.maxElements());
-
+            argBindings.put("minElements", positionalArgDef.getPositionalArgumentsAnnotation().minElements());
+            argBindings.put("maxElements", positionalArgDef.getPositionalArgumentsAnnotation().maxElements());
+            argBindings.put("collection", positionalArgDef.isCollection());
             args.get("positional").add(argBindings);
             args.get("all").add(argBindings);
         }
     }
 
-    protected void processPluginDescriptorArgument(
-            final CommandLineArgumentParser.ArgumentDefinition argDef,
-            final Map<String, Object> argBindings) {
-        if (CommandLinePluginDescriptor.class.isAssignableFrom(argDef.parent.getClass()) &&
-                CommandLineParser.getUnderlyingType(argDef.field).equals(String.class)) {
-            final CommandLinePluginDescriptor<?> descriptor = (CommandLinePluginDescriptor<?>) argDef.parent;
-            //TODO: need a way to emit a link to the index group for the plugin
-        }
-    }
-
     /**
      * Return the argument kind (required, advanced, hidden, etc) of this argumentDefinition
      *
      * @param argumentDefinition
      * @return
      */
-    private String docKindOfArg(final CommandLineArgumentParser.ArgumentDefinition argumentDefinition) {
+    private String docKindOfArg(final NamedArgumentDefinition argumentDefinition) {
 
-        // positional
+        // deprecated
         // required (common or otherwise)
         // common optional
         // advanced
         // hidden
-        // deprecated
 
         // Required first (after positional, which are separate), regardless of what else it might be
+        if (argumentDefinition.isDeprecated()) {
+            return "deprecated";
+        }
         if (argumentDefinition.isControlledByPlugin()) {
             return "dependent";
         }
-        if (!argumentDefinition.optional) {
+        if (!argumentDefinition.isOptional()) {
             return "required";
         }
-        if (argumentDefinition.isCommon) {  // these will all be optional
+        if (argumentDefinition.isCommon()) {  // these will all be optional
             return "common";
         }
-        if (argumentDefinition.field.isAnnotationPresent(Advanced.class)) {
+        if (argumentDefinition.isAdvanced()) {
             return "advanced";
         }
-        if (argumentDefinition.field.isAnnotationPresent(Hidden.class)) {
+        if (argumentDefinition.isHidden()) {
             return "hidden";
         }
-        if (argumentDefinition.field.isAnnotationPresent(Deprecated.class)) {
-            return "deprecated";
-        }
         return "optional";
     }
 
@@ -610,29 +584,7 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
      * @return
      */
     private Object prettyPrintValueString(final Object value) {
-        if (value.getClass().isArray()) {
-            final Class<?> type = value.getClass().getComponentType();
-            if (boolean.class.isAssignableFrom(type))
-                return Arrays.toString((boolean[]) value);
-            if (byte.class.isAssignableFrom(type))
-                return Arrays.toString((byte[]) value);
-            if (char.class.isAssignableFrom(type))
-                return Arrays.toString((char[]) value);
-            if (double.class.isAssignableFrom(type))
-                return Arrays.toString((double[]) value);
-            if (float.class.isAssignableFrom(type))
-                return Arrays.toString((float[]) value);
-            if (int.class.isAssignableFrom(type))
-                return Arrays.toString((int[]) value);
-            if (long.class.isAssignableFrom(type))
-                return Arrays.toString((long[]) value);
-            if (short.class.isAssignableFrom(type))
-                return Arrays.toString((short[]) value);
-            if (Object.class.isAssignableFrom(type))
-                return Arrays.toString((Object[]) value);
-            else
-                throw new RuntimeException("Unexpected array type in prettyPrintValue.  Value was " + value + " type is " + type);
-        } else if (value instanceof String) {
+        if (value instanceof String) {
             return value.equals("") ? "\"\"" : value;
         } else {
             return value.toString();
@@ -736,91 +688,171 @@ public class DefaultDocWorkUnitHandler extends DocWorkUnitHandler {
     }
 
     /**
-     * High-level entry point for creating a FreeMarker map describing the argument
-     * source with definition def, with associated javadoc fieldDoc.
+     * Process a single named argument by populating the argument bindings map for the top level freemarker map
+     * for this argument.
      *
-     * @param fieldDoc
-     * @param def
-     * @return a non-null Map binding argument keys with their values
+     * @param argBindings the argument property bindings map to be populated for this argument
+     * @param argDef the {@link NamedArgumentDefinition} for this argument
+     * @param fieldCommentText the comment text for the underlying Field for the arugment, if any
+     * @return the "kind" category for this argument as used by the freemarker template ("deprecated", "required"
+     * (common or otherwise), "common" (optional), "advanced", or "hidden")
      */
-    protected Map<String, Object> docForArgument(final FieldDoc fieldDoc, final CommandLineArgumentParser.ArgumentDefinition def) {
-        final Map<String, Object> root = new HashMap<>();
+    protected String processNamedArgument(
+            final Map<String, Object> argBindings,
+            final NamedArgumentDefinition argDef,
+            final String fieldCommentText) {
+        // Retrieve default value
+        final Object fieldValue = argDef.getArgumentValue();
+        argBindings.put("defaultValue",
+                fieldValue == null ?
+                        argDef.getDefaultValueAsString() :
+                        prettyPrintValueString(fieldValue));
 
-        final Pair<String, String> names = displayNames(def.shortName, def.getLongName());
-        root.put("name", names.getLeft());
-        root.put("synonyms", names.getRight() != null ? names.getRight() : "NA");
-        root.put("required", def.optional ? "no": "yes") ;
-        root.put("type", argumentTypeString(def.field.getGenericType()));
+        if (fieldValue instanceof Number) {
+            // Retrieve min and max / hard and soft value thresholds for numeric args
+            argBindings.put("minValue", argDef.getMinValue());
+            argBindings.put("maxValue", argDef.getMaxValue());
+            argBindings.put("minRecValue",
+                    argDef.getMinRecommendedValue() != Double.NEGATIVE_INFINITY ?
+                            argDef.getMinRecommendedValue() :
+                            "NA");
+            argBindings.put("maxRecValue",
+                    argDef.getMaxRecommendedValue() != Double.POSITIVE_INFINITY ?
+                            argDef.getMaxRecommendedValue() :
+                            "NA");
+        } else {
+            argBindings.put("minValue", "NA");
+            argBindings.put("maxValue", "NA");
+            argBindings.put("minRecValue", "NA");
+            argBindings.put("maxRecValue", "NA");
+        }
+
+        // Add in the number of times you can specify it:
+        argBindings.put("minElements", argDef.getMinElements());
+        argBindings.put("maxElements", argDef.getMaxElements());
+
+        final String kind = docKindOfArg(argDef);
+        argBindings.put("kind", kind);
+
+        final Pair<String, String> names = displayNames(argDef.getShortName(), argDef.getLongName());
+        argBindings.put("name", names.getLeft());
+        argBindings.put("synonyms", names.getRight() != null ? names.getRight() : "NA");
+        argBindings.put("required", argDef.isOptional() ? "no": "yes") ;
+        argBindings.put("type", argumentTypeString(argDef.getUnderlyingField().getGenericType()));
 
         // summary and fulltext
-        root.put("summary", def.doc != null ? def.doc : "");
-        root.put("fulltext", fieldDoc.commentText());
+        argBindings.put("summary", argDef.getDocString() != null ? argDef.getDocString() : "");
+        argBindings.put("fulltext", fieldCommentText);
 
         // Does this argument interact with any others?
-        if (def.isControlledByPlugin()) {
-            root.put("otherArgumentRequired",
-                    def.parent.getClass().getSimpleName().length() == 0 ?
-                        def.parent.getClass().getName() :
-                        def.parent.getClass().getSimpleName());
+        if (argDef.isControlledByPlugin()) {
+            argBindings.put("otherArgumentRequired",
+                    argDef.getContainingObject().getClass().getSimpleName().length() == 0 ?
+                            argDef.getContainingObject().getClass().getName() :
+                            argDef.getContainingObject().getClass().getSimpleName());
         } else {
-            root.put("otherArgumentRequired", "NA");
+            argBindings.put("otherArgumentRequired", "NA");
         }
 
-        root.put("exclusiveOf",
-                def.mutuallyExclusive != null && !def.mutuallyExclusive.isEmpty() ?
-                    String.join(", ", def.mutuallyExclusive) :
+        argBindings.put("exclusiveOf",
+                argDef.getMutexTargetList() != null && !argDef.getMutexTargetList().isEmpty() ?
+                    String.join(", ", argDef.getMutexTargetList()) :
                     "NA");
 
-        // enum options
-        root.put("options",
-                def.field.getType().isEnum() ?
-                        docForEnumArgument(def.field.getType()) :
-                        Collections.EMPTY_LIST);
+        // possible values
+        argBindings.put("options", getPossibleValues(argDef, argDef.getLongName()));
 
-        List<String> attributes = new ArrayList<>();
-        if (!def.optional) {
+        final List<String> attributes = new ArrayList<>();
+        if (!argDef.isOptional()) {
             attributes.add("required");
         }
-        if (def.field.isAnnotationPresent(Deprecated.class)) {
+        if (argDef.isDeprecated()) {
             attributes.add("deprecated");
         }
-        root.put("attributes", attributes.size() > 0 ? String.join(", ", attributes) : "NA");
+        argBindings.put("attributes", attributes.size() > 0 ? String.join(", ", attributes) : "NA");
+        argBindings.put("collection", argDef.isCollection());
+        return kind;
+    }
 
-        return root;
+    /**
+     * Return a (possibly empty) list of possible values that can be specified for this argument. Each
+     * value in the list is a map with "name" and "summary" keys.
+     * @param argDef {ArgumentDefinition}
+     * @return list of possible options for {@code argDef}. May be empty. May may not be null.
+     */
+    private List<Map<String, String>> getPossibleValues(final ArgumentDefinition argDef, final String displayName) {
+        Utils.nonNull(argDef);
+
+        final Field underlyingField = argDef.getUnderlyingField();
+        Class<?> targetClass = underlyingField.getType();
+
+        // enum options
+        if (argDef.isCollection() && underlyingField.getGenericType() instanceof ParameterizedType) {
+            // if this argument is a Collection (including an EnumSet), use the type of the generic type
+            // parameter as the target class, instead of the collection class itself
+            final Type typeParamType = underlyingField.getGenericType();
+            final ParameterizedType pType = (ParameterizedType) typeParamType;
+            final Type genericTypes[] = pType.getActualTypeArguments();
+            if (genericTypes.length != 1 || genericTypes[0] instanceof ParameterizedType) {
+                return Collections.emptyList();
+            }
+            try {
+                targetClass = Class.forName(genericTypes[0].getTypeName());
+            } catch (ClassNotFoundException e) {
+                throw new RuntimeException(String.format("No class found for type parameter (%s) used for argument (%s)",
+                        genericTypes[0].getTypeName(), displayName), e);
+            }
+        }
+
+        return targetClass.isEnum() ?
+                docForEnumArgument(targetClass) :
+                Collections.emptyList();
     }
 
     /**
      * Helper routine that provides a FreeMarker map for an enumClass, grabbing the
-     * values of the enum and their associated javadoc documentation.
+     * values of the enum and their associated javadoc or {@link CommandLineArgumentParser.ClpEnum} documentation.
      *
-     * @param enumClass
-     * @return
+     * @param enumClass an enum Class that may optionally implement {@link CommandLineArgumentParser.ClpEnum}
+     * @return a List of maps with keys for "name" and "summary" for each of the class's possible enum constants
      */
-    private List<Map<String, Object>> docForEnumArgument(final Class<?> enumClass) {
+    @SuppressWarnings("unchecked")
+    private <T extends Enum<T>> List<Map<String, String>> docForEnumArgument(final Class<?> enumClass) {
         final ClassDoc doc = this.getDoclet().getClassDocForClass(enumClass);
         if ( doc == null ) {
-            throw new RuntimeException("Tried to get docs for enum " + enumClass + " but got null instead");
+            throw new RuntimeException(String.format("Unable to get ClassDoc for enum %s", enumClass));
         }
 
-        final Set<String> enumConstantFieldNames = enumConstantsNames(enumClass);
-        final List<Map<String, Object>> bindings = new ArrayList<Map<String, Object>>();
-        for (final FieldDoc fieldDoc : doc.fields(false)) {
-            if (enumConstantFieldNames.contains(fieldDoc.name()) ) {
-                bindings.add(
-                        new HashMap<String, Object>() {
-                            static final long serialVersionUID = 0L;
-                            {
-                                put("name", fieldDoc.name());
-                                put("summary", fieldDoc.commentText());
-                            }
-                        }
-                );
+        final List<Map<String, String>> bindings = new ArrayList<>();
+        if (CommandLineArgumentParser.ClpEnum.class.isAssignableFrom(enumClass)) {
+            final T[] enumConstants = (T[]) enumClass.getEnumConstants();
+            for ( final T enumConst : enumConstants ) {
+                bindings.add(createPossibleValuesMap(
+                        enumConst.name(),
+                        ((CommandLineArgumentParser.ClpEnum) enumConst).getHelpDoc()));
+            }
+        } else {
+            final Set<String> enumConstantFieldNames = enumConstantsNames(enumClass);
+            for (final FieldDoc fieldDoc : doc.fields(false)) {
+                if (enumConstantFieldNames.contains(fieldDoc.name())) {
+                    bindings.add(createPossibleValuesMap(fieldDoc.name(), fieldDoc.commentText()));
+                }
             }
         }
 
         return bindings;
     }
 
+    private HashMap<String, String> createPossibleValuesMap(final String name, final String summary) {
+        return new HashMap<String, String>() {
+            static final long serialVersionUID = 0L;
+            {
+                put("name", name);
+                put("summary", summary);
+            }
+        };
+    }
+
     /**
      * @return a non-null set of fields that are enum constants
      */
