@@ -10,7 +10,6 @@ import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.PrintStream;
 import java.io.PrintWriter;
-import java.lang.reflect.Field;
 import java.util.*;
 import java.util.function.Consumer;
 
@@ -97,6 +96,32 @@ public final class CommandLineArgumentParserTest {
         public String OSCILLATION_FREQUENCY;
     }
 
+    @DataProvider(name="argUsageCases")
+    public Object[][] argUsageCases() {
+        return new Object[][]{
+                {
+                    new FrobnicateArguments(), Arrays.asList(
+                        "--FROBNICATION_FLAVOR <FrobnicationFlavor>",
+                        "--SHMIGGLE_TYPE <String>",
+                        "--arguments_file <File>",
+                        "--FROBNICATION_THRESHOLD,-T <Integer>",
+                        "--TRUTHINESS <Boolean>")
+                },
+                {
+                    new MixedCardinalityMutexArguments(), Arrays.asList(
+                        "--collection <String>",
+                        "--scalar <String>")
+                }
+        };
+    }
+
+    @Test(dataProvider = "argUsageCases")
+    public void testArgUsage(final Object argContainer, final List<String> expectedUsageStrings) {
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(argContainer);
+        final String usageString = clp.usage(true, false);
+        expectedUsageStrings.forEach(s -> Assert.assertTrue(usageString.contains(s)));
+    }
+
     @Test
     public void testRequiredOnlyUsage() {
         final RequiredOnlyArguments nr = new RequiredOnlyArguments();
@@ -160,6 +185,19 @@ public final class CommandLineArgumentParserTest {
         final CommandLineArgumentParser clp = new CommandLineArgumentParser(eo);
     }
 
+    class ArrayArgument {
+        // arrays have no string constructor
+        @Argument
+        public String[] stringArrayArgument = new String[] { "foo", "bar" };
+    }
+
+    @Test(expectedExceptions = CommandLineException.CommandLineParserInternalException.class)
+    public void testArrayArgument() {
+        final ArrayArgument arrayArg = new ArrayArgument();
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(arrayArg);
+        Assert.assertTrue(clp.parseArguments(System.err, new String[]{"--stringArrayArgument", "somestring"}));
+    }
+
     class AbbreviatableArgument{
         public static final String ARGUMENT_NAME = "longNameArgument";
         @Argument(fullName= ARGUMENT_NAME)
@@ -177,6 +215,19 @@ public final class CommandLineArgumentParserTest {
         clp.parseArguments(System.err, new String[]{"--" + AbbreviatableArgument.ARGUMENT_NAME.substring(0,5)});
     }
 
+    @Test
+    public void testEnsureClusteringDisabled() {
+        final String clusterOfShortArgs = "-cluster";
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(new Object());
+        final CommandLineException e = Assert.expectThrows(
+                CommandLineException.class,
+                () -> clp.parseArguments(System.err, new String[]{clusterOfShortArgs}));
+        // If clustering is enabled, the string "cluster" would be interpreted as a series of short arg names, the
+        // first one being "c", so the error message will say "-c is not...", but we want to ensure it says
+        // "-cluster is not..."
+        Assert.assertTrue(e.getMessage().startsWith(clusterOfShortArgs));
+    }
+
     @CommandLineProgramProperties(
             summary = "[oscillation_frequency]\n\nRecalibrates overthruster oscillation. \n",
             oneLineSummary = "Recalibrates the overthruster.",
@@ -285,9 +336,9 @@ public final class CommandLineArgumentParserTest {
         final CommandLineArgumentParser clp = new CommandLineArgumentParser(fo);
         Assert.assertTrue(clp.parseArguments(System.err, args));
         Assert.assertEquals(clp.getCommandLine(),
-                "FrobnicateArguments  " +
+                "FrobnicateArguments " +
                         "positional1 positional2 --FROBNICATION_THRESHOLD 17 --FROBNICATION_FLAVOR BAR " +
-                        "--SHMIGGLE_TYPE shmiggle1 --SHMIGGLE_TYPE shmiggle2 --TRUTHINESS true  --help false " +
+                        "--SHMIGGLE_TYPE shmiggle1 --SHMIGGLE_TYPE shmiggle2 --TRUTHINESS true --help false " +
                         "--version false --showHidden false");
     }
 
@@ -337,6 +388,33 @@ public final class CommandLineArgumentParserTest {
         Assert.assertEquals(fo.FROBNICATION_THRESHOLD.intValue(), 20);
     }
 
+    class FrobnicateEnumSet {
+
+        @Argument(shortName = "set-of-enum")
+        public EnumSet<FrobnicationFlavor> enumSet = EnumSet.noneOf(FrobnicationFlavor.class);
+    }
+
+    @Test
+    public void testEnumSet() {
+        final String[] args = {
+                "--set-of-enum","BAR"
+        };
+        final FrobnicateEnumSet fo = new FrobnicateEnumSet();
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(fo);
+        Assert.assertTrue(clp.parseArguments(System.err, args));
+        Assert.assertTrue(fo.enumSet.contains(FrobnicationFlavor.BAR));
+    }
+
+    @Test(expectedExceptions = CommandLineException.BadArgumentValue.class)
+    public void testEnumSetNegative() {
+        final String[] args = {
+                "--set-of-enum","NOT_AN_ENUM_MEMBER"
+        };
+        final FrobnicateEnumSet fo = new FrobnicateEnumSet();
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(fo);
+        clp.parseArguments(System.err, args);
+    }
+
     @Test(expectedExceptions = CommandLineException.MissingArgument.class)
     public void testMissingRequiredArgument() {
         final String[] args = {
@@ -556,6 +634,24 @@ public final class CommandLineArgumentParserTest {
         clp.parseArguments(System.err, args);
     }
 
+    class DanglingMutexArguments {
+        @Argument(mutex={"B"})
+        public String A;
+
+        @Argument(mutex={"nonexistent", "Z"})
+        public String B;
+
+        @Argument(mutex={"A", "B"})
+        public String Z;
+    }
+
+    @Test(expectedExceptions = CommandLineException.CommandLineParserInternalException.class)
+    public void testDanglingMutex() {
+        final DanglingMutexArguments o = new DanglingMutexArguments();
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(o);
+        Assert.assertTrue(clp.parseArguments(System.err, new String[]{"-B", "b"}));
+    }
+
     @Test
        public void testFlagNoArgument(){
         final BooleanFlags o = new BooleanFlags();
@@ -598,7 +694,16 @@ public final class CommandLineArgumentParserTest {
         Assert.assertTrue(clp.parseArguments(System.err, args));
         Assert.assertEquals(o.someNumber, 12);
         Assert.assertEquals(o.default_args.Arg1, 42);
+    }
 
+    class ArgCollectionNotInitialized {
+        @ArgumentCollection
+        public ArgsCollection default_args;
+    }
+
+    @Test(expectedExceptions = CommandLineException.CommandLineParserInternalException.class)
+    public void testArgCollectionNotInitialized() {
+        new CommandLineArgumentParser(new ArgCollectionNotInitialized());
     }
 
     class BooleanFlags{
@@ -761,31 +866,73 @@ public final class CommandLineArgumentParserTest {
         public Integer nullInteger = null;
         // Double without boundaries and null should be allowed
         @Argument(doc = "Double with null value allowed", optional = true)
-        public Double nullDouble= null;
+        public Double nullDouble = null;
+        // String null should be allowed
+        @Argument(doc = "String with null value allowed", optional = true)
+        public String nullString = null;
     }
 
     @DataProvider(name = "nullableArgs")
     public Object[][] getNullableArguments() {
         return new Object[][] {
                 // null values
-                {new String[]{}, null, null},
-                {new String[]{"--nullInteger", "null"}, null, null},
-                {new String[]{"--nullDouble", "null"}, null, null},
-                {new String[]{"--nullInteger", "null", "--nullDouble", "null"}, null, null},
+                {new String[]{}, null, null, null},
+                {new String[]{"--nullInteger", "null"}, null, null, null},
+                {new String[]{"--nullDouble", "null"}, null, null, null},
+                {new String[]{"--nullString", "null"}, null, null, null},
+                {new String[]{"--nullInteger", "null", "--nullDouble", "null", "--nullString", "null"}, null, null, null},
                 // with values
-                {new String[]{"--nullInteger", "1"}, 1, null},
-                {new String[]{"--nullDouble", "2"}, null, 2d},
-                {new String[]{"--nullInteger", "1", "--nullDouble", "2"}, 1, 2.d},
+                {new String[]{"--nullInteger", "1"}, 1, null, null},
+                {new String[]{"--nullDouble", "2"}, null, 2d, null},
+                {new String[]{"--nullString", "somestring"}, null, null, "somestring"},
+                {new String[]{"--nullInteger", "1", "--nullDouble", "2", "--nullString", "somestring"}, 1, 2.d, "somestring"},
         };
     }
 
     @Test(dataProvider = "nullableArgs")
-    public void testWithinBoundariesArguments(final String[] argv, final Integer expectedInteger, final Double expectedDouble) throws Exception {
+    public void testWithinBoundariesArguments(final String[] argv, final Integer expectedInteger, final Double expectedDouble, final String expectedString) throws Exception {
         final WithNullableArguments o = new WithNullableArguments();
         final CommandLineArgumentParser clp = new CommandLineArgumentParser(o);
         Assert.assertTrue(clp.parseArguments(System.err, argv));
         Assert.assertEquals(o.nullInteger, expectedInteger);
         Assert.assertEquals(o.nullDouble, expectedDouble);
+        Assert.assertEquals(o.nullString, expectedString);
+    }
+
+    @CommandLineProgramProperties(
+            summary = "tool with non-nullable arguments",
+            oneLineSummary = "tools with non-nullable arguments",
+            programGroup = TestProgramGroup.class
+    )
+    public class NonNullableArguments {
+        @Argument(doc = "Primitive int", optional = true)
+        public int primitiveInt;
+
+        @Argument(doc = "Primitive double", optional = true)
+        public double primitiveDouble;
+
+        @Argument(doc = "Primitive boolean", optional = true)
+        public boolean primitiveBoolean;
+    }
+
+    @DataProvider(name = "nonNullableArgs")
+    public Object[][] getNonNullableArguments() {
+        return new Object[][] {
+                // null values
+                {new String[]{"--primitiveInt", "null"}},
+                {new String[]{"--primitiveDouble", "null"}},
+                // this case triggers a very different code path; because its a flag option, its not required to
+                // have an argument value, and because the value "null" fails StrictBooleanConverter, the parser
+                // treats it is a flag with no argument value, followed by a positional argument value "null"
+                {new String[]{"--primitiveBoolean", "null"}},
+        };
+    }
+
+    @Test(dataProvider = "nonNullableArgs", expectedExceptions = CommandLineException.BadArgumentValue.class)
+    public void testSetNonNullableArgumentsToNull(final String[] args) throws Exception {
+        final NonNullableArguments o = new NonNullableArguments();
+        final CommandLineArgumentParser clp = new CommandLineArgumentParser(o);
+        Assert.assertTrue(clp.parseArguments(System.err, args));
     }
 
     @Test(expectedExceptions = CommandLineException.CommandLineParserInternalException.class)
@@ -856,7 +1003,6 @@ public final class CommandLineArgumentParserTest {
                 {new String[]{"--integerArg", "-1"}},
                 {new String[]{"--integerArg", "31"}},
                 {new String[]{"--integerArg", "106"}},
-                {new String[]{"--integerArg", "null"}},
                 {new String[]{"--doubleArg", "-1"}},
                 {new String[]{"--doubleArg", "0"}},
                 {new String[]{"--doubleArg", "21"}}
@@ -1145,18 +1291,18 @@ public final class CommandLineArgumentParserTest {
 
         // Gather all argument values of type GatherArgumentValuesTargetSuperType (or Collection<GatherArgumentValuesTargetSuperType>),
         // including subtypes.
-        List<Pair<Field, GatherArgumentValuesTargetSuperType>> gatheredArguments =
-                CommandLineParser.gatherArgumentValuesOfType(GatherArgumentValuesTargetSuperType.class, argumentSource);
+        List<Pair<ArgumentDefinition, GatherArgumentValuesTargetSuperType>> gatheredArguments =
+                clp.gatherArgumentValuesOfType(GatherArgumentValuesTargetSuperType.class);
 
         // Make sure we gathered the expected number of argument values
         Assert.assertEquals(gatheredArguments.size(), sortedExpectedGatheredValues.size(), "Gathered the wrong number of arguments");
 
         // Make sure actual gathered argument values match expected values
         List<Pair<String, String>> sortedActualGatheredArgumentValues = new ArrayList<>();
-        for ( Pair<Field, GatherArgumentValuesTargetSuperType> gatheredArgument : gatheredArguments ) {
-            Assert.assertNotNull(gatheredArgument.getKey().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
+        for ( Pair<ArgumentDefinition, GatherArgumentValuesTargetSuperType> gatheredArgument : gatheredArguments ) {
+            Assert.assertNotNull(gatheredArgument.getKey().getUnderlyingField().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
 
-            String argumentName = gatheredArgument.getKey().getAnnotation(Argument.class).fullName();
+            String argumentName = gatheredArgument.getKey().getUnderlyingField().getAnnotation(Argument.class).fullName();
             GatherArgumentValuesTargetSuperType argumentValue = gatheredArgument.getValue();
 
             sortedActualGatheredArgumentValues.add(Pair.of(argumentName, argumentValue != null ? argumentValue.getValue() : null));
@@ -1165,7 +1311,6 @@ public final class CommandLineArgumentParserTest {
 
         Assert.assertEquals(sortedActualGatheredArgumentValues, sortedExpectedGatheredValues,
                             "One or more gathered argument values not correct");
-
     }
 
     /**
@@ -1208,16 +1353,16 @@ public final class CommandLineArgumentParserTest {
 
         // Gather argument values of the raw type GatherArgumentValuesParameterizedTargetType, and make
         // sure that we match fully-parameterized declarations
-        List<Pair<Field, GatherArgumentValuesParameterizedTargetType>> gatheredArguments =
-                CommandLineParser.gatherArgumentValuesOfType(GatherArgumentValuesParameterizedTargetType.class, argumentSource);
+        List<Pair<ArgumentDefinition, GatherArgumentValuesParameterizedTargetType>> gatheredArguments =
+                clp.gatherArgumentValuesOfType(GatherArgumentValuesParameterizedTargetType.class);
 
         Assert.assertEquals(gatheredArguments.size(), 2, "Wrong number of arguments gathered");
 
-        Assert.assertNotNull(gatheredArguments.get(0).getKey().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
-        Assert.assertEquals(gatheredArguments.get(0).getKey().getAnnotation(Argument.class).fullName(), "parameterizedTypeArgument", "Wrong argument gathered");
+        Assert.assertNotNull(gatheredArguments.get(0).getKey().getUnderlyingField().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
+        Assert.assertEquals(gatheredArguments.get(0).getKey().getUnderlyingField().getAnnotation(Argument.class).fullName(), "parameterizedTypeArgument", "Wrong argument gathered");
         Assert.assertEquals(gatheredArguments.get(0).getValue().getValue(), "parameterizedTypeArgumentValue", "Wrong value for gathered argument");
-        Assert.assertNotNull(gatheredArguments.get(1).getKey().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
-        Assert.assertEquals(gatheredArguments.get(1).getKey().getAnnotation(Argument.class).fullName(), "parameterizedTypeListArgument", "Wrong argument gathered");
+        Assert.assertNotNull(gatheredArguments.get(1).getKey().getUnderlyingField().getAnnotation(Argument.class), "Gathered argument is not annotated with an @Argument annotation");
+        Assert.assertEquals(gatheredArguments.get(1).getKey().getUnderlyingField().getAnnotation(Argument.class).fullName(), "parameterizedTypeListArgument", "Wrong argument gathered");
         Assert.assertEquals(gatheredArguments.get(1).getValue().getValue(), "parameterizedTypeListArgumentValue", "Wrong value for gathered argument");
     }
 
